#include <bitset.h>
#include <macros.h>

int is_bit_set(uint8_t *set, size_t n)
{
	uint8_t *word, mask;

	word = set + n / BIT_SIZE(*set);
	mask = (1 << (n % BIT_SIZE(*set)));

	return (*word & mask) ? 1 : 0;
}

void set_bit(uint8_t *set, size_t n)
{
	uint8_t *word, mask;

	word = set + n / BIT_SIZE(*set);
	mask = (1 << (n % BIT_SIZE(*set)));

	*word |= mask;
}

void clear_bit(uint8_t *set, size_t n)
{
	uint8_t *word, mask;

	word = set + n / BIT_SIZE(*set);
	mask = (1 << (n % BIT_SIZE(*set)));

	*word &= ~mask;
}

size_t next_set_bit(uint8_t *set, size_t len)
{
	size_t n;

	for (n = 0; n < len; ++n) {
		if (is_bit_set(set, n))
			return n;
	}

	return SIZE_MAX;
}

size_t next_clear_bit(uint8_t *set, size_t len)
{
	size_t n;

	for (n = 0; n < len; ++n) {
		if (!is_bit_set(set, n))
			return n;
	}

	return SIZE_MAX;
}
